@@ -1,6 +1,6 @@
 // SPDX-License-Identifier: Apache-2.0
 /*
- * Copyright IBM Corp. 2021
+ * Copyright IBM Corp. 2021, 2024
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -55,7 +55,7 @@ typedef enum zdnn_status {
   // ----------------------------------------------------------------
   ZDNN_OK = 0x00000000, // Success.
   // ----------------------------------------------------------------
-  ZDNN_ELEMENT_RANGE_VIOLATION = ZDNN_WARNING + 0x0001, // AIU operation resulted in data that was out of the normal range.
+  ZDNN_ELEMENT_RANGE_VIOLATION = ZDNN_WARNING + 0x0001, // zAIU operation resulted in data that was out of the normal range.
   // ----------------------------------------------------------------
   ZDNN_INVALID_SHAPE = ZDNN_PARAMETER_ERROR + 0x0001, // Invalid shape information in one (or more) of the input/output tensor(s).
   ZDNN_INVALID_LAYOUT,                                // Invalid layout information in one (or more) of the input/output tensor(s).
@@ -67,12 +67,20 @@ typedef enum zdnn_status {
   ZDNN_INVALID_STRIDES,                               // Invalid stride height or width parameter.
   ZDNN_MISALIGNED_PARMBLOCK,                          // NNPA parameter block is not on double word boundary.
   ZDNN_INVALID_CLIPPING_VALUE,                        // Invalid clipping for the specified operation.
+  ZDNN_INVALID_ADJUSTMENT_FACTOR,                     // Invalid adjustment for the specified operation.
+  ZDNN_INVALID_EPSILON,                               // Invalid epsilon for the specified operation.
+  ZDNN_INVALID_TRANSFORM_TYPE,                        // Invalid transformation type
+  ZDNN_INVALID_BETA,                                  // Invalid beta value for the specified operation.
+  ZDNN_INVALID_GAMMA,                                 // Invalid gamma value for the specified operation.
+  ZDNN_INVALID_BESSEL_CORRECTION,                     // Invalid bessel correction value for the specified operation.
+  ZDNN_INVALID_SCALE,                                 // Invalid scale value for the specified operation.
+  ZDNN_INVALID_OFFSET,                                // Invalid offset value for the specified operation.
   // ----------------------------------------------------------------
   ZDNN_ALLOCATION_FAILURE = ZDNN_DATA_ERROR + 0x0001, // Can not allocate storage.
   ZDNN_INVALID_BUFFER,                                // Buffer address is NULL or not on 4K-byte boundary, or insufficient buffer size.
   ZDNN_CONVERT_FAILURE,                               // Floating point data conversion failure.
   ZDNN_INVALID_STATE,                                 // Invalid zTensor state.
-  ZDNN_UNSUPPORTED_AIU_EXCEPTION,                     // AIU operation returned an unexpected exception.
+  ZDNN_UNSUPPORTED_AIU_EXCEPTION,                     // zAIU operation returned an unexpected exception.
   // ----------------------------------------------------------------
   ZDNN_UNSUPPORTED_PARMBLOCK = ZDNN_HW_ERROR + 0x0001, // NNPA parameter block format is not supported by the model.
   ZDNN_UNAVAILABLE_FUNCTION,                           // Specified NNPA function is not defined or installed on the machine.
@@ -114,40 +122,58 @@ typedef enum nnpa_function_code {
   NNPA_MAX = 21,
   NNPA_LOG = 32,
   NNPA_EXP = 33,
+  NNPA_SQRT = 34,
+  NNPA_INVSQRT = 35,
   // reserved = 48
   NNPA_RELU = 49,
   NNPA_TANH = 50,
   NNPA_SIGMOID = 51,
   NNPA_SOFTMAX = 52,
+  NNPA_GELU = 53,
   NNPA_BATCHNORMALIZATION = 64,
+  NNPA_MOMENTS = 65,
+  NNPA_LAYERNORM = 66,
+  NNPA_NORM = 67,
   NNPA_MAXPOOL2D = 80,
   NNPA_AVGPOOL2D = 81,
   NNPA_LSTMACT = 96,
   NNPA_GRUACT = 97,
   NNPA_CONVOLUTION = 112,
   NNPA_MATMUL_OP = 113,
-  NNPA_MATMUL_OP_BCAST23 = 114
+  NNPA_MATMUL_OP_BCAST23 = 114,
+  NNPA_MATMUL_OP_BCAST1 = 115,
+  NNPA_TRANSFORM = 240,
+  NNPA_REDUCE = 241
 } nnpa_function_code;
 
 typedef enum nnpa_parmblk_format {
   NNPA_PARMBLKFORMAT_0 = 0,
+  NNPA_PARMBLKFORMAT_1 = 1,
 } nnpa_parmblk_format;
 
-typedef enum nnpa_data_type { NNPA_DATATYPE_1 = 0 } nnpa_data_type;
+typedef enum nnpa_data_type {
+  NNPA_DATATYPE_1 = 0,
+  NNPA_32_BIT_BINARY_FP_SHORT = 6,
+  NNPA_8_BIT_BINARY_INT = 8,
+  NNPA_32_BIT_BINARY_INT = 10
+} nnpa_data_type;
 
 typedef enum nnpa_layout_format {
   NNPA_LAYOUTFMT_4DFEATURE = 0,
-  NNPA_LAYOUTFMT_4DKERNEL = 1
+  NNPA_LAYOUTFMT_4DKERNEL = 1,
+  NNPA_LAYOUTFMT_4DWEIGHTS = 2,
+  NNPA_LAYOUTFMT_4DGENERIC = 31
 } nnpa_layout_format;
 
 typedef enum nnpa_bfp_format {
   // 0 is reversed
   NNPA_BFPFMT_TINY = 1,
-  NNPA_BFPFMT_SHORT = 2,
+  NNPA_BFPFMT_SHORT = 2
 } nnpa_bfp_format;
 
-// NNPA_SOFTMAX work area sizes required by the NNPA hardware
+// NNPA_SOFTMAX, NNPA_REDUCE, and NNPA_TRANSFORM require 8K work area
 #define ZDNN_SOFTMAX_SAVEAREA_SIZE 8 * 1024
+#define ZDNN_8K_SAVEAREA_SIZE 8 * 1024
 
 // NNPA Hardware defined values for Function Specific Parameters
 typedef enum nnpa_matmul_operations {
@@ -161,7 +187,13 @@ typedef enum nnpa_matmul_operations {
 } nnpa_matmul_operations;
 
 typedef enum nnpa_matmul_bcast_operations {
-  NNPA_MATMUL_BCAST_OP_ADDITION = 0
+  NNPA_MATMUL_BCAST_OP_ADDITION = 0,
+  NNPA_MATMUL_BCAST_OP_COMP_HIGH = 1,
+  NNPA_MATMUL_BCAST_OP_COMP_NOT_LOW = 2,
+  NNPA_MATMUL_BCAST_OP_COMP_EQUAL = 3,
+  NNPA_MATMUL_BCAST_OP_COMP_NOT_EQUAL = 4,
+  NNPA_MATMUL_BCAST_OP_COMP_NOT_HIGH = 5,
+  NNPA_MATMUL_BCAST_OP_COMP_LOW = 6
 } nnpa_matmul_bcast_operations;
 
 typedef enum nnpa_softmax_act {
@@ -169,6 +201,13 @@ typedef enum nnpa_softmax_act {
   NNPA_SOFTMAX_LOG = 1
 } nnpa_softmax_act;
 
+typedef enum nnpa_reduce_operations {
+  NNPA_REDUCE_OP_MINIMUM = 0,
+  NNPA_REDUCE_OP_MINIMUM_IDX = 1,
+  NNPA_REDUCE_OP_MAXIMUM = 2,
+  NNPA_REDUCE_OP_MAXIMUM_IDX = 3
+} nnpa_reduce_operations;
+
 // -----------------------------------------------------------------------------
 // zdnn_query_*() bit-field enums
 // -----------------------------------------------------------------------------
@@ -177,12 +216,17 @@ typedef enum nnpa_softmax_act {
 #define MSB_BITMASK(field_size, pos) 1u << ((field_size - 1) - pos)
 
 typedef enum zdnn_query_datatypes {
-  QUERY_DATATYPE_INTERNAL1 = MSB_BITMASK(16, NNPA_DATATYPE_1)
+  QUERY_DATATYPE_INTERNAL1 = MSB_BITMASK(16, NNPA_DATATYPE_1),
+  QUERY_DATATYPE_BINARY_FP32 = MSB_BITMASK(16, NNPA_32_BIT_BINARY_FP_SHORT),
+  QUERY_DATATYPE_BINARY_INT8 = MSB_BITMASK(16, NNPA_8_BIT_BINARY_INT),
+  QUERY_DATATYPE_BINARY_INT32 = MSB_BITMASK(16, NNPA_32_BIT_BINARY_INT)
 } zdnn_query_datatypes;
 
 typedef enum zdnn_query_layoutfmts {
   QUERY_LAYOUTFMT_4DFEATURE = MSB_BITMASK(32, NNPA_LAYOUTFMT_4DFEATURE),
-  QUERY_LAYOUTFMT_4DKERNEL = MSB_BITMASK(32, NNPA_LAYOUTFMT_4DKERNEL)
+  QUERY_LAYOUTFMT_4DKERNEL = MSB_BITMASK(32, NNPA_LAYOUTFMT_4DKERNEL),
+  QUERY_LAYOUTFMT_4DWEIGHTS = MSB_BITMASK(32, NNPA_LAYOUTFMT_4DWEIGHTS),
+  QUERY_LAYOUTFMT_4DGENERIC = MSB_BITMASK(32, NNPA_LAYOUTFMT_4DGENERIC)
 } zdnn_query_layoutfmts;
 
 typedef enum zdnn_query_bfpfmts {
@@ -196,9 +240,17 @@ typedef enum zdnn_query_bfpfmts {
 
 typedef enum zdnn_data_types {
   ZDNN_DLFLOAT16 = NNPA_DATATYPE_1, // 16-bit deep learning format
-  BFLOAT = 253,                     // Brain floating point format
-  FP16 = 254,                       // 16-bit IEEE-754 floating point format
-  FP32 = 255,                       // 32-bit IEEE-754 floating point format
+  ZDNN_BINARY_FP32 =
+      NNPA_32_BIT_BINARY_FP_SHORT, // 32-bit binary-floating-point format
+  ZDNN_BINARY_INT8 =
+      NNPA_8_BIT_BINARY_INT, // 8-bit signed or unsigned binary integer
+  ZDNN_BINARY_INT32 =
+      NNPA_32_BIT_BINARY_INT, // 32-bit signed or unsigned binary integer
+  INT8 = 251,                 // 8-bit signed or unsigned binary integer format
+  INT32 = 252,                // 32-bit signed or unsigned binary integer format
+  BFLOAT = 253,               // Brain floating point format
+  FP16 = 254,                 // 16-bit IEEE-754 floating point format
+  FP32 = 255,                 // 32-bit IEEE-754 floating point format
 } zdnn_data_types;
 
 typedef enum zdnn_data_layouts {
@@ -221,11 +273,21 @@ typedef enum zdnn_data_layouts {
 
 typedef enum zdnn_data_formats {
   ZDNN_FORMAT_4DFEATURE =
-      NNPA_LAYOUTFMT_4DFEATURE, // tensor in AIU data layout format 0
+      NNPA_LAYOUTFMT_4DFEATURE, // tensor in zAIU data layout format 0
   ZDNN_FORMAT_4DKERNEL =
-      NNPA_LAYOUTFMT_4DKERNEL, // tensor in AIU data layout format 1
+      NNPA_LAYOUTFMT_4DKERNEL, // tensor in zAIU data layout format 1
+  ZDNN_FORMAT_4DWEIGHTS =
+      NNPA_LAYOUTFMT_4DWEIGHTS, // tensor in zAIU data layout format 2
+  ZDNN_FORMAT_4DGENERIC =
+      NNPA_LAYOUTFMT_4DGENERIC, // tensor in zAIU data layout 31
 } zdnn_data_formats;
 
+typedef enum zdnn_quantized_transform_types {
+  QUANTIZED_DLFLOAT16 = 0,   // quantized dlfloat16
+  QUANTIZED_INT8 = 1,        // quantized int8
+  QUANTIZED_WEIGHTS_INT8 = 2 // quantized weights
+} zdnn_quantized_transform_types;
+
 // Supported padding types for use in pooling functions
 typedef enum zdnn_pool_padding {
   VALID_PADDING = 0,
@@ -245,7 +307,14 @@ typedef enum zdnn_matmul_ops {
 
 // Support operations for use in matmul function
 typedef enum zdnn_matmul_bcast_ops {
-  MATMUL_BCAST_OP_ADDITION = NNPA_MATMUL_BCAST_OP_ADDITION
+  MATMUL_BCAST_OP_ADDITION = NNPA_MATMUL_BCAST_OP_ADDITION,
+  MATMUL_BCAST_OP_GREATER = NNPA_MATMUL_BCAST_OP_COMP_HIGH,
+  MATMUL_BCAST_OP_GREATER_EQUAL = NNPA_MATMUL_BCAST_OP_COMP_NOT_LOW,
+  MATMUL_BCAST_OP_EQUAL = NNPA_MATMUL_BCAST_OP_COMP_EQUAL,
+  MATMUL_BCAST_OP_NOT_EQUAL = NNPA_MATMUL_BCAST_OP_COMP_NOT_EQUAL,
+  MATMUL_BCAST_OP_LESSER_EQUAL = NNPA_MATMUL_BCAST_OP_COMP_NOT_HIGH,
+  MATMUL_BCAST_OP_LESSER = NNPA_MATMUL_BCAST_OP_COMP_LOW
+
 } zdnn_matmul_bcast_ops;
 
 typedef enum zdnn_softmax_act {
@@ -258,6 +327,19 @@ typedef enum zdnn_conv2d_act {
   CONV2D_ACT_RELU
 } zdnn_conv2d_act;
 
+// Support operations for use in reduce functions
+typedef enum zdnn_reduce_ops {
+  REDUCE_OP_MINIMUM = NNPA_REDUCE_OP_MINIMUM,
+  REDUCE_OP_MINIMUM_IDX = NNPA_REDUCE_OP_MINIMUM_IDX,
+  REDUCE_OP_MAXIMUM = NNPA_REDUCE_OP_MAXIMUM,
+  REDUCE_OP_MAXIMUM_IDX = NNPA_REDUCE_OP_MAXIMUM_IDX
+} zdnn_reduce_ops;
+
+typedef enum zdnn_moments_bessel {
+  MOMENTS_BESSEL_POPULATION,
+  MOMENTS_BESSEL_SAMPLE,
+} zdnn_moments_bessel;
+
 // -----------------------------------------------------------------------------
 // Structs
 // ----------------------------------------------------------------------------
@@ -282,13 +364,16 @@ typedef struct zdnn_ztensor {
   uint64_t buffer_size;               // tensor size in bytes
   void *buffer;                       // pointer to the tensor in memory
   bool is_transformed; // indicator if data in buffer has been transformed
-  char reserved[31];   // not currently used, should contain zeros.
+  char reserved[3];    // not currently used, should contain zeros.
+  float rec_scale;    // the scale factor for quantization, stored as reciprocal
+  float offset;       // the offset for quantization
+  char reserved2[20]; // not currently used, should contain zeros.
 } zdnn_ztensor;
 
-#define ZDNN_VERSION "1.0.1"
-#define ZDNN_VERNUM 0x010001 // 0x[major][minor][patch]
+#define ZDNN_VERSION "1.1.1"
+#define ZDNN_VERNUM 0x010101 // 0x[major][minor][patch]
 #define ZDNN_VER_MAJOR 1
-#define ZDNN_VER_MINOR 0
+#define ZDNN_VER_MINOR 1
 #define ZDNN_VER_PATCH 1
 
 // -----------------------------------------------------------------------------
@@ -328,6 +413,11 @@ zdnn_status
 zdnn_generate_transformed_desc(const zdnn_tensor_desc *pre_tfrmd_desc,
                                zdnn_tensor_desc *tfrmd_desc);
 
+zdnn_status zdnn_generate_quantized_transformed_desc(
+    const zdnn_tensor_desc *pre_tfrmd_desc,
+    zdnn_quantized_transform_types transform_type,
+    zdnn_tensor_desc *tfrmd_desc);
+
 zdnn_status zdnn_generate_transformed_desc_concatenated(
     const zdnn_tensor_desc *pre_tfrmd_desc, zdnn_concat_info info,
     zdnn_tensor_desc *tfrmd_desc);
@@ -338,14 +428,27 @@ zdnn_status zdnn_free_ztensor_buffer(const zdnn_ztensor *ztensor);
 void zdnn_init_ztensor(zdnn_tensor_desc *pre_tfrmd_desc,
                        zdnn_tensor_desc *tfrmd_desc, zdnn_ztensor *output);
 
+void zdnn_init_quantized_ztensor(zdnn_tensor_desc *pre_tfrmd_desc,
+                                 zdnn_tensor_desc *tfrmd_desc, float scale,
+                                 float offset, zdnn_ztensor *output);
+
 zdnn_status zdnn_init_ztensor_with_malloc(zdnn_tensor_desc *pre_tfrmd_desc,
                                           zdnn_tensor_desc *tfrmd_desc,
                                           zdnn_ztensor *output);
 
+zdnn_status zdnn_init_quantized_ztensor_with_malloc(
+    zdnn_tensor_desc *pre_tfrmd_desc, zdnn_tensor_desc *tfrmd_desc, float scale,
+    float offset, zdnn_ztensor *output);
+
+bool zdnn_is_quantized_ztensor(zdnn_ztensor *ztensor);
+
 void zdnn_reset_ztensor(zdnn_ztensor *ztensor);
 
 uint64_t zdnn_getsize_ztensor(const zdnn_tensor_desc *tfrmd_desc);
 
+zdnn_status zdnn_getrange_ztensor(const zdnn_ztensor *ztensor, float *min,
+                                  float *max);
+
 // -----------------------------------------------------------------------------
 // External Query Functions
 // -----------------------------------------------------------------------------
@@ -359,6 +462,7 @@ bool zdnn_is_nnpa_conversion_installed(nnpa_data_type type,
                                        uint16_t format_bitmask);
 
 uint32_t zdnn_get_nnpa_max_dim_idx_size();
+uint32_t zdnn_get_max_for_dim(uint8_t dimension);
 uint64_t zdnn_get_nnpa_max_tensor_size();
 
 zdnn_status zdnn_refresh_nnpa_query_result();
@@ -389,6 +493,9 @@ zdnn_status zdnn_max(const zdnn_ztensor *input_a, const zdnn_ztensor *input_b,
 
 zdnn_status zdnn_log(const zdnn_ztensor *input, zdnn_ztensor *output);
 zdnn_status zdnn_exp(const zdnn_ztensor *input, zdnn_ztensor *output);
+zdnn_status zdnn_sqrt(const zdnn_ztensor *input, zdnn_ztensor *output);
+zdnn_status zdnn_invsqrt(const zdnn_ztensor *input, float epsilon,
+                         zdnn_ztensor *output);
 
 // -----------------------------------------------------------------------------
 // External Activation Operations
@@ -396,10 +503,17 @@ zdnn_status zdnn_exp(const zdnn_ztensor *input, zdnn_ztensor *output);
 
 zdnn_status zdnn_relu(const zdnn_ztensor *input, const void *clipping_value,
                       zdnn_ztensor *output);
+zdnn_status zdnn_leaky_relu(const zdnn_ztensor *input,
+                            const void *clipping_value, float adjustment_factor,
+                            zdnn_ztensor *output);
 zdnn_status zdnn_tanh(const zdnn_ztensor *input, zdnn_ztensor *output);
 zdnn_status zdnn_sigmoid(const zdnn_ztensor *input, zdnn_ztensor *output);
 zdnn_status zdnn_softmax(const zdnn_ztensor *input, void *save_area,
                          zdnn_softmax_act act_func, zdnn_ztensor *output);
+zdnn_status zdnn_softmax_mask(const zdnn_ztensor *input, void *save_area,
+                              zdnn_softmax_act act_func, uint32_t softmax_mask,
+                              zdnn_ztensor *output);
+zdnn_status zdnn_gelu(const zdnn_ztensor *input, zdnn_ztensor *output);
 
 // -----------------------------------------------------------------------------
 // Recurrent Neural Network (RNN) Operations
@@ -434,6 +548,17 @@ zdnn_status zdnn_matmul_bcast_op(const zdnn_ztensor *input_a,
                                  const zdnn_ztensor *input_c,
                                  zdnn_matmul_bcast_ops op_type,
                                  zdnn_ztensor *output);
+zdnn_status zdnn_matmul_transpose_op(const zdnn_ztensor *input_a,
+                                     const zdnn_ztensor *input_b,
+                                     const zdnn_ztensor *input_c,
+                                     bool transpose_a, bool transpose_b,
+                                     zdnn_matmul_ops op_type,
+                                     zdnn_ztensor *output);
+zdnn_status zdnn_quantized_matmul_op(
+    const zdnn_ztensor *input_a, const zdnn_ztensor *input_b,
+    const zdnn_ztensor *input_c, zdnn_matmul_ops op_type, const int8_t clip_min,
+    const int8_t clip_max, const bool disable_clipping, const bool dequantize,
+    const bool pre_computed, void *work_area, zdnn_ztensor *output);
 
 // -----------------------------------------------------------------------------
 // External Norm Operations
@@ -442,9 +567,21 @@ zdnn_status zdnn_matmul_bcast_op(const zdnn_ztensor *input_a,
 zdnn_status zdnn_batchnorm(const zdnn_ztensor *input_a,
                            const zdnn_ztensor *input_b,
                            const zdnn_ztensor *input_c, zdnn_ztensor *output);
-
+zdnn_status zdnn_norm(const zdnn_ztensor *input_a, const zdnn_ztensor *input_b,
+                      zdnn_ztensor *output);
+zdnn_status zdnn_moments(const zdnn_ztensor *input,
+                         zdnn_moments_bessel bessel_correction_type,
+                         zdnn_ztensor *output_a, zdnn_ztensor *output_b);
+zdnn_status zdnn_layernorm(const zdnn_ztensor *input_a,
+                           const zdnn_ztensor *input_b,
+                           const zdnn_ztensor *input_c, const float beta_value,
+                           const float gamma_value, const float epsilon_value,
+                           zdnn_ztensor *output);
 zdnn_status zdnn_meanreduce2d(const zdnn_ztensor *input, zdnn_ztensor *output);
 
+zdnn_status zdnn_reduce(const zdnn_ztensor *input, void *save_area,
+                        zdnn_reduce_ops op_type, zdnn_ztensor *output);
+
 // -----------------------------------------------------------------------------
 // External Pool Operations
 // -----------------------------------------------------------------------------
@@ -477,6 +614,13 @@ zdnn_status zdnn_conv2d(const zdnn_ztensor *input, const zdnn_ztensor *kernel,
 
 zdnn_status zdnn_transform_ztensor(zdnn_ztensor *ztensor, ...);
 
+zdnn_status zdnn_transform_ztensor_with_saturation(zdnn_ztensor *ztensor, ...);
+
+zdnn_status zdnn_transform_quantized_ztensor(zdnn_ztensor *ztensor,
+                                             bool saturation_control,
+                                             int8_t clip_min, int8_t clip_max,
+                                             const void *data);
+
 zdnn_status zdnn_transform_origtensor(const zdnn_ztensor *ztensor,
                                       void *out_buf);
 
@@ -495,4 +639,15 @@ uint32_t zdnn_get_library_version();
 
 const char *zdnn_get_status_message(zdnn_status status);
 
+// -----------------------------------------------------------------------------
+// zDNN Data Type Limit Functions
+// -----------------------------------------------------------------------------
+
+zdnn_status zdnn_get_max_limit(zdnn_data_types transformed_type,
+                               zdnn_data_types pre_transformed_type,
+                               void *limit);
+zdnn_status zdnn_get_min_limit(zdnn_data_types transformed_type,
+                               zdnn_data_types pre_transformed_type,
+                               void *limit);
+
 #endif /* ZDNN_ZDNN_H_ */
